#
# Some extended information about the module
#
# Copyright (C) 2004 nick knouf <nknouf@desnak.com>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: entries.py,v 1.6 2004/10/12 00:58:51 commit Exp $

__author__ = "Nick Knouf <nknouf@mimeme.net>"
__version__ = "$Revision: 1.6 $"
__date__ = "$Date: 2004/10/12 00:58:51 $"             
__copyright__ = "Copyright (c) 2004 desnak"
__license__ = "GPL2"

# python standard library imports
import os
import sys
import time

# ddd specific imports
from ddd.__init__ import ddd_schema_location
from ddd.factories.xmllibFactory import xmllibFactory
from ddd.exceptions import *

class entries(object):
    """The `entries` class provides an amalgamation of the sets of `projectEntry`, `thoughtsEntry`, `codeEntry`, and `designEntry` classes.  Prior to creating these sets of classes, it reads in a ddd file and parses it.

    More information to come."""

    def __init__(self, filename=None, xmllib=None):
        """If given `filename`, read in the file and pass it off to our parser function.

        More to come."""

        if xmllib is None:
            raise NoXmllibError, "Need a xmllib instance to continue."

        # create our xml library instance
        self.xmllib = xmllib
       
        # list of xpath expressions
        self.project_xpath = "/ddd/projectEntry"
        self.design_xpath = "/ddd/designEntry"
        self.thoughts_xpath = "/ddd/thoughtsEntry"
        self.code_xpath = "/ddd/codeEntry"

        # store the node lists
        self.project_nodes = []
        self.design_nodes = []
        self.thoughts_nodes = []
        self.code_nodes = []

        if filename:
            self.read_file(filename)

        #self.project_nodes = self.xmllib.xpath(self.xml, projects_xpath)
        #self.design_nodes = self.xmllib.xpath(self.xml, design_xpath)
        #self.thoughts_nodes = self.xmllib.xpath(self.xml, thoughts_xpath)
        #self.code_nodes = self.xmllib.xpath(self.xml, code_xpath)

    def read_file(self, filename):
        """Read `filename` and populate our list of entry nodes."""

        self.current_filename = filename

        # before we parse, make sure we're valid
        # i hate global variables just as much as the next guy, but in this
        # case i need to use it so the functions know where the schema is
        # located (usually somewhere under "/usr/share" or "/usr/local/share"
        global ddd_schema_location
        valid = self.xmllib.validate(ddd_schema_location, filename)
        if not valid:
            raise InvalidDocumentError, "The given document does not validate against the ddd schema."

        xml = self.xmllib.parse(filename)

        # parse the metadata
        self.metadata = self.parse_metadata(xml)

        # add all of the different types of nodes to the lists
        project_nodelist = self.xmllib.xpath(xml, self.project_xpath)
        for node in project_nodelist:
            self.project_nodes.append(entry.create(type="projectEntry", node=node, xmllib=self.xmllib))

        design_nodelist = self.xmllib.xpath(xml, self.design_xpath)
        for node in design_nodelist:
            self.design_nodes.append(entry.create(type="designEntry", node=node, xmllib=self.xmllib))

        thoughts_nodelist = self.xmllib.xpath(xml, self.thoughts_xpath)
        for node in thoughts_nodelist:
            self.thoughts_nodes.append(entry.create(type="thoughtsEntry", node=node, xmllib=self.xmllib))

        code_nodelist = self.xmllib.xpath(xml, self.code_xpath)
        for node in code_nodelist:
            self.code_nodes.append(entry.create(type="codeEntry", node=node, xmllib=self.xmllib))

    def save_entries(self, filename=None):
        """Serialize the `*Entry` and `metadata` nodes to file."""

        if filename is None:
            filename = self.current_filename

        # okay, create a new document and add the root element
        doc = self.xmllib.newDoc()
        root = self.xmllib.newNode(name='ddd')
        self.xmllib.setRoot(doc, root)
       
        # let's add our entries from the lists as children
        for node in self.project_nodes:
            self.xmllib.addChild(parent=root, child=node.entry)
        for node in self.code_nodes:
            self.xmllib.addChild(parent=root, child=node.entry)
        for node in self.thoughts_nodes:
            self.xmllib.addChild(parent=root, child=node.entry)
        for node in self.design_nodes:
            self.xmllib.addChild(parent=root, child=node.entry)

        # add the metadata node
        self.xmllib.addChild(parent=root, child=self.metadata_node)

        # get the serialized string and save to `filename`
        output_string = self.xmllib.serializeToFile(doc, filename)

        # free our document
        self.xmllib.free(doc)

        # return success
        return True

    def parse_metadata(self, xml):
        """Parse the `metadata` section of the `xml` tree; results are returned as a dictionary."""

        self.metadata = {}

        # get the title
        title_node = self.xmllib.xpath(xml, "/ddd/metadata/title")
        check_nodelist_length(title_node, "/ddd/metadata/title", no_duplicates=True)
        self.metadata["title"] = self.xmllib.getContent(title_node[0])
        
        # get the authors
        author_nodes = self.xmllib.xpath(xml, "/ddd/metadata/author/person/name")
        check_nodelist_length(author_nodes, "/ddd/metadata/author/person/name", no_duplicates=False)
        self.metadata["authors"] = []
        for node in author_nodes:
            self.metadata["authors"].append(self.xmllib.getContent(node))

        # get the legals
        legal_nodes = self.xmllib.xpath(xml, "/ddd/metadata/legal/para")
        check_nodelist_length(legal_nodes, "/ddd/metadata/legal/para", no_duplicates=False)
        self.metadata["legal"] = []
        for node in legal_nodes:
            self.metadata["legal"].append(self.xmllib.getContent(node))

        # get ourselves the `metadata` node by itself and save it
        self.metadata_node = self.xmllib.xpath(xml, "/ddd/metadata")
        check_nodelist_length(self.metadata_node, "/ddd/metadata", no_duplicates=True)
        self.metadata_node = self.xmllib.copyNode(self.metadata_node[0])

        return self.metadata

    def free(self):
        """Free all of the nodes that we created."""

        for node in self.project_nodes:
            self.xmllib.freeNode(node.entry)
        for node in self.design_nodes:
            self.xmllib.freeNode(node.entry)
        for node in self.code_nodes:
            self.xmllib.freeNode(node.entry)
        for node in self.thoughts_nodes:
            self.xmllib.freeNode(node.entry)

class entry(object):
    """Base class that provides an interface to working with entries at a low level."""

    def __init__(self, entry_node, create=False, xmllib_name='libxml2'):
        """Initialize the class with the given entry_node."""
        self.entry = entry_node

        self.xmllib = xmllibFactory.create(xmllib_name)
        self.xmllib = self.xmllib()

        self.entryTitle = None
        self.keywords = []
        self.content = []
        self.id = None
        self.created = None
        self.modified = None

        # check if some of the data is already populated; if so, perform the
        # `get_*` methods to set the instance variables
        #entryTitle_nodes = self.xmllib.xpath(self.entry, "entryTitle")
        #check_nodelist_length(entryTitle_nodes, "entryTitle", no_duplicates=True)
        #if self.xmllib.getContent(entryTitle_nodes[0]) is not '':
        #    self.get_entryTitle()
        #    self.get_keywords()
        #    self.get_content()

    def create(type="projectEntry", node=None, entryTitle=None, keywords=[], content=[], xmllib=None):
        """Create a new entry object with the given values; also creates the xml node."""

        # were we given a node to begin with?
        if node is None: 
            given_node = None
        else:
            given_node = node

        if xmllib is None:
            raise NoXmllibError, "Need a xmllib instance to continue."

        if given_node is None:
            node = xmllib.newNode(name=type)
    
            # create empty values
            xmllib.newChild(parent=node, name="entryTitle", content='')
            xmllib.newChild(parent=node, name="keywords", content='')
            xmllib.newChild(parent=node, name="para", content='')
    
            creation_time = str(time.time())
    
            # set the attributes
            xmllib.newProp(node, "id", creation_time)
            xmllib.newProp(node, "created", creation_time)
            xmllib.newProp(node, "modified", creation_time)
    
            # get the correct xpath level
            # @@HACK@@
            doc = xmllib.newDoc()
            xmllib.setRoot(doc, node)
            node = xmllib.xpath(doc, type)[0]

        # create a new object based on the given type
        entries_module = sys.modules[entries.__module__]
        subclass = "%s" % type
        entry_class = hasattr(entries_module, subclass) and getattr(entries_module, subclass)
        entry_object = entry_class(node)

        if given_node is None:
            entry_object.id = creation_time
            entry_object.created = creation_time
            entry_object.modified = creation_time
    
            entry_object.set_entryTitle(entryTitle)
            entry_object.set_keywords(keywords)
            entry_object.set_content(content)

        return entry_object
       
    create = staticmethod(create)

    def get_entryTitle(self):
        """Return the `entryTitle` from the `entry_node`; update the instance variable `self.entryTitle` if necessary."""

        if self.entryTitle is None:
            entryTitle_nodes = self.xmllib.xpath(self.entry, "entryTitle")
            check_nodelist_length(entryTitle_nodes, "entryTitle", no_duplicates=True)

            # if no exceptions are raised, we only have one entryTitle node
            # and we take the first one and get its content
            self.entryTitle = self.xmllib.getContent(entryTitle_nodes[0])

            return self.entryTitle
        else:
            return self.entryTitle

    def set_entryTitle(self, entryTitle):
        """Set the entryTitle in the entry node to `entryTitle`; update the instance variable."""

        entryTitle_nodes = self.xmllib.xpath(self.entry, "entryTitle")
        check_nodelist_length(entryTitle_nodes, "entryTitle", no_duplicates=True)

        # update the entryTitle content
        self.xmllib.setContent(entryTitle_nodes[0], entryTitle)

        # update the instance variable
        self.entryTitle = entryTitle

        # update the modified time
        self.modified = time.time()

        return self.entryTitle

    def get_keywords(self):
        """Return the `keywords` list from the `entry_node`; update the instance variable `self.keywords` if necessary."""

        if self.keywords == []:
            keywords_nodes = self.xmllib.xpath(self.entry, "keywords")
            check_nodelist_length(keywords_nodes, "keywords", no_duplicates=True)

            # if no exceptions are raised, we only have one keywords node
            # we now try and split the data and strip the results
            keywords_string = self.xmllib.getContent(keywords_nodes[0])
            self.keywords = keywords_string.split(',')

            for keyword in self.keywords:
                keyword.strip()

            return self.keywords
        else:
            return self.keywords

    def set_keywords(self, keywords):
        """Set the keywords in the entry node to `keywords`; update the instance variable."""

        keywords_nodes = self.xmllib.xpath(self.entry, "keywords")
        check_nodelist_length(keywords_nodes, "keywords", no_duplicates=True)

        # make our list into a string
        keywords_string = ", ".join(keywords)

        # update the keywords content
        self.xmllib.setContent(keywords_nodes[0], keywords_string)

        # update the instance variable
        self.keywords = keywords

        # update the modified time
        self.modified = time.time()

        return self.keywords

    def get_content(self):
        """Return the `content` list from the `entry_node`; update the instance variable `self.content` if necessary."""

        if self.content == []:
            content_nodes = self.xmllib.xpath(self.entry, "para")
            check_nodelist_length(content_nodes, "content", no_duplicates=False)

            # if no exceptions are raised, we should have at least one para
            # node.  we loop through the nodes and append it to our content list

            for para in content_nodes:
                self.content.append(self.xmllib.getContent(para))

            return self.content
        else:
            return self.content

    def set_content(self, paras):
        """Set the `para` nodes from the `entry_node` with the list of paragraphs in `paras`; update the instance variable `self.content` if necessary."""
        
        # get our list of para nodes and remove them
        content_nodes = self.xmllib.xpath(self.entry, "para")
        check_nodelist_length(content_nodes, "content", no_duplicates=False)

        # if no exceptions are raised, we should have at least one para
        # node.  let's remove it
        for node in content_nodes:
            node.freeNode()

        # add the new para nodes to the entry
        for para in paras:
            self.xmllib.newChild(parent=self.entry, name="para", content=para)
            #self.entry.newTextChild(None, "para", para)

        self.content = paras

        # update the modified time
        self.modified = time.time()

        return self.content

    def get_attributes(self):
        """Return a dictionary of attributes from the `entry_node`; update the instance variables if necessary."""

        if self.id is None:
            id_nodes = self.xmllib.xpath(self.entry, "@id")
            check_nodelist_length(id_nodes, "@id", no_duplicates=True)

            # if no exceptions are raised, we only have one id node
            # lets get its content
            self.id = self.xmllib.getContent(id_nodes[0])

        if self.created is None:
            created_nodes = self.xmllib.xpath(self.entry, "@created")
            check_nodelist_length(created_nodes, "@created", no_duplicates=True)

            # if no exceptions are raised, we only have one created node
            # lets get its content
            self.created = self.xmllib.getContent(created_nodes[0])

        if self.modified is None:
            modified_nodes = self.xmllib.xpath(self.entry, "@modified")
            check_nodelist_length(modified_nodes, "@modified", no_duplicates=True)

            # if no exceptions are raised, we only have one modified node
            # lets get its content
            self.modified = self.xmllib.getContent(modified_nodes[0])

        return {"id":self.id, "created":self.created, "modified":self.modified}

class projectEntry(entry):
    """A class for working with a projectEntry."""

    def __init__(self, entry_node):
        super(projectEntry, self).__init__(entry_node)

class thoughtsEntry(entry):
    """A class for working with a thoughtsEntry."""

    def __init__(self, entry_node):
        super(thoughtsEntry, self).__init__(entry_node)

class codeEntry(entry):
    """A class for working with a codeEntry."""

    def __init__(self, entry_node):
        super(codeEntry, self).__init__(entry_node)

class designEntry(entry):
    """A class for working with a designEntry."""

    def __init__(self, entry_node):
        super(designEntry, self).__init__(entry_node)

def check_nodelist_length(nodelist, expression, no_duplicates=False):
    """Helper function to perform routine checks of the nodelist length after evaluation of an xPath expression."""

    if len(nodelist) == 0:
        raise NoNodesFoundError, "No nodes match the xPath expression %s" % expression 

    if no_duplicates:
        if len(nodelist) > 1:
            raise MultipleNodesError, "Multiple nodes were returned by the exPath expression %s, but I excepted only one result." % expression
