#!env python
"""
An abstract factory to validate the xml documents for the site.

This depends on xmllibFactory.py found in this distribution.  In theory, this should work for any xml processing library (and that implements the interface developed here), but it has only been tested with libxml2 and RELAX NG.
"""

__author__ = "Nick Knouf <nknouf@desnak.com>"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2004/08/23 22:07:22 $"
__copyright__ = "Copyright 2004, desnak"
##__license__ = "Python"

# standard modules
import sys

# our modules
from xmllibFactory import xmllibFactory

# abstract Validator class
class Validator(object):
	"""Base Validator class.  All factory objects extend this class.  In order to write interfaces for other xml schema languages, you need to subclass Validator and implement *at least* the methods that raise NotImplementedError."""

	def __init__(self, schema='', document=''):
		self.isValid = 0

	def validate(self, document):
		raise NotImplementedError


class relaxngValidator(Validator):
	"""Validate a document using the RELAX NG schema language (http://www.relaxng.org).  This schema language is much easier to use (and read) than XML Schema and is becoming the XML schema language of choice.

	Currently, this class uses the libxml2 python bindings to provide validation support through the xmllibFactory.  This could be extended to include other bindings through code that implements the abstract interface in xmllibFactory.py
	"""

	def __init__(self, schema='', document=''):
		super(relaxngValidator, self).__init__(schema, document)

		xmllib = xmllibFactory().create('libxml2')
		self.xmllib = xmllib()

		self.reader = self.setTextReader(document)
		self.rng_parser = self.getRelaxNGParser(schema)

	def setTextReader(self, document):
		"""Set our text reader."""

		self.reader = self.xmllib.getTextReader(document)

		return self.reader

	def getRelaxNGParser(self,schema):
		"""Setup our relax NG parser object."""

		self.rng_parser = self.xmllib.getRelaxNGParser(schema)

		return self.rng_parser

	def validate(self, document='', schema=''):
		"""Validate document 'document' against the schema 'schema'."""

		if not self.reader:
			self.setTextReader(document)

		if not self.rng_parser:
			self.getRelaxNGParser(schema)

		if self.reader is not None and self.rng_parser is not None:
			self.reader.RelaxNGSetSchema(self.rng_parser)

			return_value = self.reader.Read()
			while return_value == 1:
				return_value = reader.Read()

			if self.reader.IsValid() == 1:
				self.isValid = 1
				return 1
			else:
				self.isValid = 0
				return 0

		return 0

class ValidatorFactory(object):
	"""Create a Validator object from the factory options."""

	def __init__(self):
		pass

	def create(self, factory_name):
		"""Create a Validatory object from factory 'factory_name'."""

		module = sys.modules[ValidatorFactory.__module__]

		subclass = "%sValidator" % factory_name

		return hasattr(module, subclass) and getattr(module, subclass)
