# some copyright info someday
"""
Abstract factory for creating an interface to a specific rdf library.
"""
__author__ = "Nick Knouf <nknouf@mit.edu>"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2004/08/23 22:07:22 $"
__copyright__ = "Copyright (c) 2004 MIT"
#__license__ = "Python"

# Import system modules
import sys

# Import the necessary user-defined modules
try:
    import logging as log
except:
    pass

class rdflib(object):
	"""'rdflib' is the abstract base class that all implementation specific rdf library classes must subclass.  At the minimum, subclasses must implement the functions that raise "NotImplementedError"."""

	def __init__(self):
		pass

	def createModel(self, storage):
		raise NotImplementedError

	def createStorage(self, storageName, *arg, **kw):
		raise NotImplementedError

	def addStatement(self, model, statement):
		raise NotImplementedError

	def createParser(self, type=''):
		raise NotImplementedError

	def createStatement(self, subject, predicate, object):
		raise NotImplementedError

	def createNode(self, uri):
		raise NotImplementedError

	def getStatements(self, model, statement):
		raise NotImplementedError

	def createNamespace(self, uri):
		raise NotImplementedError

class RedlandLibrary(rdflib):
	"""Give an interface to the Redland framework.  Returns a RDF model."""

	import RDF
	import libxml2, libxslt

	storage_types = {'hashes':('hashes', "new='yes', 'hash-type='memory', dir='.'")}

	def __init__(self, storage_name='hashes'):
		import RDF
		super(RedlandLibrary, self).__init__()
		
		log.debug("initialzed a RedlandLibrary object")
		self.RDF = RDF

	def createParser(self, type='raptor'):
		"""Create a RDF parser object and save it as an class variable."""

		log.debug("created a parser based on %s" % type)
		self.parser = self.RDF.Parser(type)

		return self.parser

	def createNamespace(self, uri):
		"""Create a namespace object for defining user-specific nodes."""

		log.debug("created a namespace object")
		return self.RDF.NS(uri)

	def createNode(self, uri):
		"""Create a node object, commonly for use in creating statement objects."""

		return self.RDF.Node(uri)

	def createStatement(self, subject=None, predicate=None, object=None):
		"""Create a statement with any of the parameters: subject, predicate, object.  Each of the input parameters is a node of type RDF.Node.  The return value is a RDF.Statement object.

		The default is to create the empty statement using nodes with empty literals."""

		# take care of defaults
		if subject is None:
			subject = self.RDF.Node('')
		
		if predicate is None:
			predicate = self.RDF.Node('')
		
		if object is None:
			object = self.RDF.Node('')

		# create that statement like we want
		log.info("created the statement with the following nodes: subject=%s; predicate=%s; object=%s" % str(subject), str(predicate), str(object))
		return self.RDF.Statement(subject, predicate, object)

	def createModel(self, storage=None):
		"""Create an RDF model with the specified storage."""

		if not self.storage:
			if storage is None:
				log.critical("You need to provide *some* sort of storage for the model.")
				raise ConfluxExceptions.NeedStorageToMakeModelError, \
						"You need to provide *some* sort of storage for \
						the model."
			else:
				self.model = self.RDF.Model(storage)
		else:
			self.model = self.RDF.Model(self.storage)

		log.debug("created a model")
		return self.model

class rdflibFactory(object): 
	"""Factory to create a specific rdflib object."""

	def create(self, factoryName):
		"""Create the desired rdflib from a given name.
    
Found the method to do this from the excellent book, available online, DiveIntoP
ython (http://diveintopython.org)
		"""

		module = sys.modules[rdflibFactory.__module__]

		subclass = "%sLibrary" % factoryName

		log.debug("trying to create the '%s' rdflib from the factory" % factoryName)
		return hasattr(module, subclass) and getattr(module, subclass)
