#
# Some extended information about the module
#
# Copyright (C) 2004 nick knouf <nknouf@desnak.com>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: io.py,v 1.3 2004/09/30 20:37:45 commit Exp $

# python standard library
import time

# ddd specific
from factories.xmllibFactory import xmllibFactory

class io(object):
    """The io class of ddd handles all of the basic input/output of ddd files."""
    
    def __init__(self, xmllib='libxml2', filename=None):
        """Initialization function that (optionally) returns an xml object"""

        # create our xmllib class
        xmllib = xmllibFactory.create(xmllib)

        # make an instance of this class
        self.xmllib = xmllib()

        if filename is not None:
            self.xml = self.xmllib.parse(filename)

    def read_ddd_file(self, filename):
        """Reads a ddd xml file and returns a reference to the xml object"""

        if filename is not None:
            self.xml = self.xmllib.parse(filename)

        return self.xml

    def edit_entry(self, entryType, entryID, **kw):
        """Edit an entry of type entryType and ID entryID

        **kw is list of key=value pairs that follow the format for ddd dictionary entries.
        
        Yes, that's vague."""

        # no error correcting yet!
        for key, value in kw:
            self.ddd[entryType][entryID][key] = value

        # update modified time
        self.ddd[entryType][entryID]["modified"] = time.time()

    def add_entry(self, entryType, **kw):
        """Add a new entry of type entryType.

        **kw is list of key=value pairs that follow the format for ddd dictionary entries.
        
        Yes, that's vague."""
        
        # no error correcting yet!
        id = time.time()
        for key, value in kw:
            self.ddd[entryType][id][key] = value

        self.ddd[entryType][entryID]["created"] = id
        self.ddd[entryType][entryID]["modified"] = id

    def parse_ddd(self, xml):
        """Parses a ddd xml object and creates a python dictionary of references to *Entry objects."""

        self.ddd = ddd_dict()

        self.ddd["metadata"] = self.getMetadata(xml)

        self.ddd["projectEntry"] = self.getEntries(xml, "projectEntry")
        self.ddd["codeEntry"] = self.getEntries(xml, "codeEntry")
        self.ddd["designEntry"] = self.getEntries(xml, "designEntry")
        self.ddd["thoughtsEntry"] = self.getEntries(xml, "thoughtsEntry")

        return self.ddd

    def getMetadata(self, xml):
        """Given a xml document, read the "metadata" section and create a dictionary containing the results."""

        metadata_xpath = "/ddd/metadata"

        metadata = {}

        # we assume only one metadata node!
        node = self.xmllib.xpath(xml, metadata_xpath)[0]

        metadata["title"] = self.xmllib.xpath(node, "title/text()")[0].content
        metadata["name"] = self.xmllib.xpath(node, "author/person/name/text()")[0].content

        paras = self.xmllib.xpath(node, "legal/para")
        metadata["legal"] = []
        for para in paras:
            metadata["legal"].append(para.content)

        return metadata

    def getEntries(self, xml, entryType):
        """Get a list of entries from the xml object for a given entryType."""

        entry_xpath = "/ddd/%s" % entryType
        entryTitle_xpath = "/entryTitle/text()"
        keywords_xpath = "/keywords/text()"
        para_xpath = "/para/*"
        entry = {}

        nodes = self.xmllib.xpath(xml, entry_xpath)

        if nodes is not []:
            for node in nodes:
                # get the id value
                id = self.xmllib.xpath(node, '@id')[0].content

                # setup the dict
                entry[id] = {}

                # get the created value
                entry[id]["created"] = self.xmllib.xpath(node, '@created')[0].content

                # get the modified value
                entry[id]["modified"] = self.xmllib.xpath(node, '@modified')[0].content

                # get the entryTitle
                entry[id]["entryTitle"] = self.xmllib.xpath(node, 'entryTitle/text()')[0].content

                # get the keywords 
                entry[id]["keywords"] = self.xmllib.xpath(node, 'keywords/text()')[0].content

                # loop through each paragraph
                entry[id]["para"] = []
                paras = self.xmllib.xpath(node, 'para')
                for para in paras:
                    entry[id]["para"].append(para.content)

            return entry
        else:
            return []

class ddd_dict(dict):
    """Derived dictionary class that only allows certain values for its first-level keys."""

    allowed_keys = ('metadata', 'projectEntry', 'thoughtsEntry', \
                    'codeEntry', 'designEntry')

    def __init__(self):
        dict.__init__(self)

        for key in self.allowed_keys:
            dict.__setitem__(self, key, {})            

    def __getitem__(self, key):
        if not key in self.allowed_keys:
            raise "Invalid key"
        else:
            return dict.__getitem__(self, key)
