#!env python
"""
some information about the module
"""

__author__ = "Nick Knouf <nknouf@desnak.com>"
__version__ = "$Revision: 1.3 $"
__date__ = "$Date: 2004/10/11 04:53:11 $"
__copyright__ = "Copyright 2004, desnak"
##__license__ = "Python"

import unittest
from ddd import entries
from ddd.factories.xmllibFactory import xmllibFactory

class EntryTest(unittest.TestCase):
    
    def setUp(self):

        self.xmllib = xmllibFactory.create('libxml2')
        self.xmllib = self.xmllib()

        # read in and parse our testing xml file
        self.xml = self.xmllib.parse('test_data/test.xml')

        # pick a representative entry node to test
        self.projects_xpath = "/ddd/projectEntry"
        self.entry = self.xmllib.xpath(self.xml, self.projects_xpath)[0]
        self.entry = entries.entry(self.entry)

    def testHasClasses(self):
        """Does the module have the classes we want?"""
        self.assert_(hasattr(entries, "entry"), "'entries' module does not have a class 'entry'; bad news!!!")
        self.assert_(hasattr(entries, "projectEntry"), "'entries' module does not have a class 'projectEntry'; bad news!!!")
        self.assert_(hasattr(entries, "thoughtsEntry"), "'entries' module does not have a class 'thoughtsEntry'; bad news!!!")
        self.assert_(hasattr(entries, "codeEntry"), "'entries' module does not have a class 'codeEntry'; bad news!!!")
        self.assert_(hasattr(entries, "designEntry"), "'entries' module does not have a class 'designEntry'; bad news!!!")

    def testGetValues(self):
        """Do we get values from the entry properly?  Do we then have non-null instance varaibles?"""

        entry = self.entry

        # test for null instance variables
        self.assertEqual(entry.entryTitle, None, "'entry' instance does not start with entryTitle as None.")
        self.assertEqual(entry.keywords, [], "'entry' instance does not start with keywords as [].")
        self.assertEqual(entry.content, [], "'entry' instance does not start with content as [].")

        # test for proper result from getting, setting functions
        self.assertEqual(entry.get_entryTitle(), "test projectEntry", "'entry' instance does not return the correct entryTitle.")
        self.assertEqual(entry.get_keywords(), ['project'], "'entry' instance does not return the correct keywords list.")
        self.assertEqual(entry.get_content(), ['a test projectEntry'], "'entry' instance does not return the correct content.")

        # test for proper setting of instance variables
        self.assertEqual(entry.entryTitle, "test projectEntry", "'entry' instance does not return the correct entryTitle.")
        self.assertEqual(entry.keywords, ['project'], "'entry' instance does not return the correct keywords list.")
        self.assertEqual(entry.content, ['a test projectEntry'], "'entry' instance does not return the correct content.")

        # test for proper getting of attributes
        self.assertEqual(entry.get_attributes(), {"id":'1096315709.3698609', "created":"1096315709.3698609", "modified":"1096315709.3698609"}, "`entry` instance does not return the correct attributes of the entry.")

        # test for proper setting of instance variables
        self.assertEqual(entry.id, "1096315709.3698609", "`entry` instance did not set the correct id variable.")
        self.assertEqual(entry.created, "1096315709.3698609", "`entry` instance did not set the correct `created` variable.")
        self.assertEqual(entry.modified, "1096315709.3698609", "`entry` instance did not set the correct `modified` variable.")

    def testSetValues(self):
        """Can we properly set the necessary values?"""

        entry = self.entry

        entry.set_entryTitle("foo")
        entry.set_keywords(["foobar", "barfoo"])
        entry.set_content(["this is a test", "we are the champions, my friend."])

        # test the instance variables
        self.assertEqual(entry.entryTitle, "foo", "`entry` instance failed to update the `entryTitle` instance variable.")
        self.assertEqual(entry.keywords, ["foobar", "barfoo"], "`entry` instance failed to update the `keywords` instance variable.")
        self.assertEqual(entry.content, ["this is a test", "we are the champions, my friend."], "`entry` instance failed to update the `content` instance variable.")

        # begin: corner cases
        # what about only one keyword?
        entry.set_keywords(["foo"])
        self.assertEqual(entry.keywords, ["foo"], "`entry` instance failed to update the `keywords` instance variable.")
        # end: corner cases

    def testNewObject(self):
        """Can we create a new entry object with the desired elements?"""

        entry = self.entry.create(type="projectEntry", entryTitle="new", keywords=["testing", "foo"], content=["this is a test.", "this should work."], xmllib=self.xmllib) 

        # test the set attributes
        self.assertEqual(entry.entryTitle, "new", "created `entry` instance has an incorrect `entryTitle` instance variable.")
        self.assertEqual(entry.keywords, ["testing", "foo"], "created `entry` instance has an incorrect `keywords` instance variable.")
        self.assertEqual(entry.content, ["this is a test.", "this should work."], "created `entry` instance has an incorrect `content` instance variable.")

class EntriesTest(unittest.TestCase):
    
    def setUp(self):
        self.xmllib = xmllibFactory.create('libxml2')
        self.xmllib = self.xmllib()

        self.entries = entries.entries(filename='test_data/test.xml', xmllib=self.xmllib)

    def testGetEntries(self):
        """Can we get our set of test entries?"""
       
        entries = self.entries

        # first check if we have the correct number of elements in our lists
        self.assertEqual(len(entries.project_nodes), 1, "`entries` instance does not have the correct number of `projectEntry` nodes.")
        self.assertEqual(len(entries.design_nodes), 1, "`entries` instance does not have the correct number of `designEntry` nodes.")
        self.assertEqual(len(entries.thoughts_nodes), 1, "`entries` instance does not have the correct number of `thoughtsEntry` nodes.")
        self.assertEqual(len(entries.code_nodes), 1, "`entries` instance does not have the correct number of `codeEntry` nodes.")

        # all right, let's try and get some of our content
        projectEntry_object = entries.project_nodes[0]
        self.assertEqual(projectEntry_object.get_entryTitle(), "test projectEntry", "`projectEntry` instance does not return the correct `entryTitle`.")
        self.assertEqual(projectEntry_object.get_keywords(), ["project"], "`projectEntry` instance does not return the correct `keywords`.")
        self.assertEqual(projectEntry_object.get_content(), ["a test projectEntry"], "`projectEntry` instance does not return the correct `content`.")

        thoughtsEntry_object = entries.thoughts_nodes[0]
        self.assertEqual(thoughtsEntry_object.get_entryTitle(), "test thoughtsEntry", "`thoughtsEntry` instance does not return the correct `entryTitle`.")
        self.assertEqual(thoughtsEntry_object.get_keywords(), ["thoughts"], "`thoughtsEntry` instance does not return the correct `keywords`.")
        self.assertEqual(thoughtsEntry_object.get_content(), ["a test thoughtsEntry"], "`thoughtsEntry` instance does not return the correct `content`.")

        codeEntry_object = entries.code_nodes[0]
        self.assertEqual(codeEntry_object.get_entryTitle(), "test codeEntry", "`codeEntry` instance does not return the correct `entryTitle`.")
        self.assertEqual(codeEntry_object.get_keywords(), ["code"], "`codeEntry` instance does not return the correct `keywords`.")
        self.assertEqual(codeEntry_object.get_content(), ["a test codeEntry"], "`codeEntry` instance does not return the correct `content`.")

        designEntry_object = entries.design_nodes[0]
        self.assertEqual(designEntry_object.get_entryTitle(), "test designEntry", "`designEntry` instance does not return the correct `entryTitle`.")
        self.assertEqual(designEntry_object.get_keywords(), ["design"], "`designEntry` instance does not return the correct `keywords`.")
        self.assertEqual(designEntry_object.get_content(), ["a test designEntry"], "`designEntry` instance does not return the correct `content`.")

    def testSaveEntries(self):
        """Do we properly save our entries?"""

        # we have to a helper function since the unittest framework doesn't
        # allow us to test assertions in a way i'd like
        filename = 'test_data/you_can_delete_me.xml'
        self.entries.save_entries(filename)
        file_exist = self._testFileExists(filename)

        self.assert_(file_exist, "`save_entries` method failed to create the correct file (%s)." % filename)

    def testGotMetadata(self):
        """Do we get the correct metadata?"""

        self.assertEqual(self.entries.metadata["title"], "ddd", "`entries` object failed to properly parse the metadata (wrong `title`).")
        self.assertEqual(self.entries.metadata["authors"], ["nick knouf"], "`entries` object failed to properly parse the metadata (wrong `authors` list).")
        self.assertNotEqual(self.entries.metadata["legal"], [], "`entries` object failed to properly parse the metadata (wrong `legal` information).")

    def _testFileExists(self, filename):
        """Helper method to test for whether or not a file is written by the`save_entries` method of the `entries` class."""

        import os
        try:
            os.stat(filename)
            return True
        except OSError:
            return False

if __name__ == "__main__":
    unittest.main()
