#!env python
"""
Code to test the xmllibFactory code 
"""

__author__ = "Nick Knouf <nknouf@desnak.com>"
__version__ = "$Revision: 1.4 $"
__date__ = "$Date: 2004/10/12 00:56:53 $"
__copyright__ = "Copyright 2004, desnak"
##__license__ = "Python"

import unittest
import xmllibFactory

functions = ['parse', 'getChildren', 'getRoot', 'xpath', 'free']

xml_document = """
<!-- <?xml version="1.0" encoding="UTF-8"?> -->
<root>
    <foo>
        <bar id="foobar" />
    </foo>
    <test1 id="123">
        <test2>only a test</test2>
        <test3>Only a test with an <test4>interior</test4> test</test3>
    </test1>
</root>
"""

# the relaxNG examples are from the relaxNG tutorial:
# http://www.oasis-open.org/committees/relax-ng/tutorial.html
relaxNG_schema = """<grammar xmlns="http://relaxng.org/ns/structure/1.0">

  <start>
    <ref name="AddressBook"/>
  </start>

  <define name="AddressBook">
    <element name="addressBook">
      <zeroOrMore>
        <ref name="Card"/>
      </zeroOrMore>
    </element>
  </define>

  <define name="Card">
    <element name="card">
      <ref name="Name"/>
      <ref name="Email"/>
    </element>
  </define>

  <define name="Name">
    <element name="name">
      <text/>
    </element>
  </define>

  <define name="Email">
    <element name="email">
      <text/>
    </element>
  </define>

</grammar>"""

relaxNG_document = """<addressBook>
  <card>
    <name>John Smith</name>
    <email>js@example.com</email>
  </card>
  <card>
    <name>Fred Bloggs</name>
    <email>fb@example.net</email>
  </card>
</addressBook>"""

class xmllibTest(unittest.TestCase):
    """Test the xmllib code"""

    def setUp(self):
        self.xml = xmllibFactory.xmllib()

    def testHasFunctions(self):
        """Does our xmllib factory object have the desired functions?"""
        
        for function in functions:
            self.assert_(hasattr(self.xml, function), "Unable to find the desired function '%s'." % function)

    def testReadFile(self):
        """Does the _readFile private function work as expected?"""

        self.assertEquals(self.xml._readFile(''), None, '_readFile does not return "None" for an empty filename.')

class libxml2ProductBasicTest(unittest.TestCase):
    """Test the libxml2Product code."""

    def setUp(self):
        xmllib = xmllibFactory.xmllibFactory().create('libxml2')
        self.xml = xmllib()

    def testLibxml2ProductExists(self):
        """Did we create what we thought we created?"""

        self.assertEquals(self.xml.__class__, xmllibFactory.libxml2Product, "Didn't create the libxml2 product that we thought we would.")

    def testParse(self):
        """Does the parse function work as expected?"""

        self.assertEquals(self.xml.parse(''), None, "'parse' function doesn't work as expected.")

        

class libxml2ProductExtendedTest(unittest.TestCase):
    """Test some extended parts of the libxml2 wrapper code."""

    def setUp(self):
        xmllib = xmllibFactory.xmllibFactory().create('libxml2')
        self.xml = xmllib()

        self.doc = self.xml.parse(xml_document, type='string')

    def testExtendedParse(self):
        """Do we parse a real document correctly?"""

        self.assert_(hasattr(self.doc, 'getRootElement'), "Unable to get the correct document type from parsing an XML fragment.")

    def testGetRoot(self):
        """Do we get the correct root element?"""
    
        root = self.xml.getRoot(self.doc)
        self.assert_(hasattr(root, 'get_name'), "Unable to get a node object when using the 'getRoot' function.")

    def testGetName(self):
        """Do we get the correct name of an element?"""

        root = self.xml.getRoot(self.doc)

        self.assertEquals(root.get_name(), 'root', "Unable to get the correct name of a node.""")

    def testSimpleXPath(self):
        """Does a simple xpath expression give us the correct result?"""

        xPathExpressions = ["//test4/text()", "/root/test1/test3/test4/text()"]

        for expression in xPathExpressions:
            node = self.xml.xpath(self.doc, expression)

            self.assertEquals(node[0].content, 'interior', "Unable to get the correct text value with a simple xPath expression")

    def testValidate(self):
        """Do we propery validate a document?"""

        # test validating files
        result = self.xml.validate('test_data/relaxNG_schema.rng', 'test_data/relaxNG_document.xml')
        self.assert_(result, "`validate` method failed to properly validate a relaxNG schema and xml document stored in file.")

        # test validating strings
        result = self.xml.validate(relaxNG_schema, relaxNG_document, schema_format="string", document_format="string")
        self.assert_(result, "`validate` method failed to properly validate a relaxNG document and schema in a string.")

        # test validating combination
        result = self.xml.validate(relaxNG_schema, 'test_data/relaxNG_document.xml', schema_format="string")
        self.assert_(result, "`validate` method failed to properly validate a relaxNG schema (as a string) and xml document stored in file.")
        result = self.xml.validate('test_data/relaxNG_schema.rng', relaxNG_document, document_format="string")
        self.assert_(result, "`validate` method failed to properly validate a relaxNG schema and xml document stored in a string.")


    def testAdvancedXPath(self):
        """Do we get the correct values for more complicated xPath expressions?"""

        xPathExpressions = ["/root/test1[@id='123']/test2/text()", \
                            "//descendant::test2/text()", \
                            "//child::test1/*/text()"]

        for expression in xPathExpressions:
            node = self.xml.xpath(self.doc, expression)

            self.assertEquals(node[0].content, 'only a test', "Unable to get the correct text value with a simple xPath expression")

if __name__ == "__main__":
    unittest.main()
